#!/bin/sh
#
# Converting messageflow configuration after upgrade to EIM PB 5.1
#
# Converts MsgFlow table by:
#    Replacing OBJTYPE 'r' (REPORT) with
#          'D' (DIMSE-N) if TARGET is DICOM device
#          'F' (Final)   if TARGET is HL7 device
#    Replacing OBJTYPE 'o' (ORDER) with 'N' (New) and
#              add 3 more rows with OBJTYPE 'U' (Updated), 'M' (Completed), and 'C' (Canceled)
#
# Also convert report.cfg and order.cfg configuration files, by replacing 'sendTo' devices to
# device[<event>,...]|... format
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

. $HOME/etc/setenv.sh

LOGGER="/usr/bin/logger"
TAG="iface/convert51[$$]"

# Logs error message in error log and print it to
# standard error
#
# $1 - error message
#
error_()
{
	echo "$1" >&2
	$LOGGER -p $LOG_FACILITY.err -t "$TAG" "$1"
}


# Converts OBJTYPE into event
#
# $1 - OBJTYPE
#
type2event_()
{
	if [ "$1" = "D" ] ; then
		echo "DIMSE_N_REPORT_EVENTS"
	elif [ "$1" = "P" ] ; then
		echo "HL7_REPORT_PRELIMINARY"
	elif [ "$1" = "F" ] ; then
		echo "HL7_REPORT_APPROVED"
	elif [ "$1" = "N" ] ; then
		echo "HL7_ORDER_CREATED"
	elif [ "$1" = "U" ] ; then
		echo "HL7_ORDER_UPDATED"
	elif [ "$1" = "M" ] ; then
		echo "HL7_ORDER_COMPLETED"
	elif [ "$1" = "C" ] ; then
		echo "HL7_ORDER_CANCELED"
	fi
}

# Creates configuration file
#
# $1 -  the list of device, message type pairs
#       devices and message types should be separated by '#'
#       Eg. dev1#type1 dev2#type2 ...
createConfig_()
{
	[ -z "$1" ] && return 1

	valuelist=`sed -e "s;\(.\);,'\1';g" -e 's;^.;;' <<<"$1"`
	inClause="OBJTYPE IN ( $valuelist )"
	devsrc=`echo "select distinct SOURCE from $MSGFLOWTBL where ${inClause};" | \
			$MYSQL_HOME/bin/mysql -N $DBNAME`

	if [ $? -ne 0 ] ; then
		error_ "MySQL error occured while converting order configuration file"
		return 1
	fi

	for device in $devsrc ; do
		type="-"
		host="-"
		port="-"
		ae="-"
		vmd="-"
		targets="-"
		if [ "$device" != "LOCAL" ] ; then
			SQL_RES=`echo "select HOST, PORT, TYPE, AE from $TARGETTBL where ID='$device';" | \
					$MYSQL_HOME/bin/mysql -E $DBNAME 2>/dev/null`

			if [ $? -ne 0 ] ; then
				error_ "MySQL error occured while getting device info for $device"
				return 1
			fi

			# This line below will create DEV_<DNAME>="<VALUE>" equations and
			# evaluate them, so eg.: $DEV_HOST and DEV_AE can be used in later codes
			eval `echo "$SQL_RES" | grep -v "^\*\*" | sed -e 's/ */DEV_/' -e 's/: /="/' -e 's/$/"/'`

			[ -n "$DEV_HOST" ] && host="$DEV_HOST"
			[ -n "$DEV_PORT" ] && port="$DEV_PORT"
			[ -n "$DEV_TYPE" ] && type="$DEV_TYPE"
			[ -n "$DEV_AE" ] && ae="$DEV_AE"

			[ "$DEV_TYPE" = "H" ] && vmd="${HL7_VMD_DIR}/${device}.vmd"
		fi

		SQL_RES=`echo "select concat(TARGET, '#', OBJTYPE) from $MSGFLOWTBL where SOURCE='$device' AND ${inClause} order by TARGET;" | \
				$MYSQL_HOME/bin/mysql -N $DBNAME 2>/dev/null`

		if [ $? -ne 0 ] ; then
			error_ "MySQL error occured while getting message flow info for $device"
			return 1
		fi

		dev=""
		targetlist=""
		for targetinfo in $SQL_RES ; do
			targetdev=`cut -f 1 -d "#" <<<"$targetinfo"`
			targettype=`cut -f 2 -d "#" <<<"$targetinfo"`

			event=`type2event_ $targettype`

			if [ -z "$event" ] ; then
				error_ "Unknown type $targettype for source ($device) - target ($targetdev) (skipped)"
				continue;
			fi

			if [ "$dev" != "$targetdev" ] ; then
				[ -n "$targetlist" ] && targetlist="$targetlist]|"
				targetlist="${targetlist}${targetdev}[${event}"
				dev="$targetdev"
			else
				targetlist="${targetlist},${event}"
			fi
		done

		if [ -n "$targetlist" ] ; then
			targets="$targetlist]"
		fi

		echo "$device $type $host $port $ae $vmd $targets"
	done
}

$LOGGER -p $LOG_FACILITY.debug -t "$TAG" "Converting messageflow configuration after upgrade to EIM PB 5.1 started"

# Fix the database first

SQL_RES=`echo "select distinct SOURCE, TARGET,OBJTYPE,TYPE from $MSGFLOWTBL, $TARGETTBL where TARGET=ID AND binary OBJTYPE IN ('r', 'o');" | \
		$MYSQL_HOME/bin/mysql -N $DBNAME`
if [ $? -ne 0 ] ; then
	error_ "MySQL error occured while converting MessageFlow configuration"
	exit 1
fi

if [ -z "$SQL_RES" ] ; then
	# Nothing to do
	exit 0
fi

$LOGGER -p $LOG_FACILITY.info -t "$TAG" "Converting messageflow configuration after upgrade to EIM PB 5.1 started"

echo "$SQL_RES" | while read source target event type ; do
	if [ "$event" = "r" ] ; then
		newEvent=""
		[ "$type" = 'D' ] && newEvent="D"
		[ "$type" = 'H' ] && newEvent="F"
		if [ -z "$newEvent" ] ; then
			error_ "Error calculating new Report OBJTYPE for device $target [$type]"
			exit 1
		fi
		echo "update $MSGFLOWTBL set OBJTYPE='$newEvent' where SOURCE='$source' AND TARGET='$target' AND OBJTYPE='r';" | \
				$MYSQL_HOME/bin/mysql -N $DBNAME
		if [ $? -ne 0 ] ; then
			error_ "MySQL error occured while converting Report MessageFlow configuration for device $target"
			exit 1
		fi
	elif [ "$event" = "o" ] ; then
		cat <<-ENDOFMYSQLCMD | $MYSQL_HOME/bin/mysql -N $DBNAME
			update $MSGFLOWTBL set OBJTYPE='N' where SOURCE='$source' AND TARGET='$target' AND OBJTYPE='o';
			INSERT INTO $MSGFLOWTBL (SOURCE, TARGET, OBJTYPE) VALUES ('$source', '$target', 'U');
			INSERT INTO $MSGFLOWTBL (SOURCE, TARGET, OBJTYPE) VALUES ('$source', '$target', 'C');
			INSERT INTO $MSGFLOWTBL (SOURCE, TARGET, OBJTYPE) VALUES ('$source', '$target', 'M');
		ENDOFMYSQLCMD

		if [ $? -ne 0 ] ; then
			error_ "MySQL error occured while converting Order MessageFlow configuration for device $target"
			exit 1
		fi
	else
		error_ "Unsupported event: $event (skipped)"
	fi
done

if [ $? -ne 0 ] ; then
	exit 1
fi


# Fixing the configuration files

# Creating order config file
createConfig_ NUCM > ${IFACE_ORDER_CONFIG_FILE}.tmp

if [ $? -ne 0 ] ; then
	rm -f ${IFACE_ORDER_CONFIG_FILE}.tmp
	exit 1
fi

[ -f "${IFACE_ORDER_CONFIG_FILE}" ] && mv ${IFACE_ORDER_CONFIG_FILE} ${IFACE_ORDER_CONFIG_FILE}.convert51
mv ${IFACE_ORDER_CONFIG_FILE}.tmp ${IFACE_ORDER_CONFIG_FILE}


# Creating report config file
createConfig_ DPF > ${IFACE_REPORT_CONFIG_FILE}.tmp

if [ $? -ne 0 ] ; then
	rm -f ${IFACE_REPORT_CONFIG_FILE}.tmp
	exit 1
fi

[ -f "${IFACE_REPORT_CONFIG_FILE}" ] && mv ${IFACE_REPORT_CONFIG_FILE} ${IFACE_REPORT_CONFIG_FILE}.convert51
mv ${IFACE_REPORT_CONFIG_FILE}.tmp ${IFACE_REPORT_CONFIG_FILE}

$LOGGER -p $LOG_FACILITY.info -t "$TAG" "Converting messageflow configuration after upgrade to EIM PB 5.1 finished"

exit 0
